@@ -1105,21 +1105,6 @@ tlbflush:
 
 out:
 
-    if ( rc < 0 && ( op == INSERT || op == ALLOCATE ) &&
-         addr != start_gpaddr )
-    {
-        BUG_ON(addr == end_gpaddr);
-        /*
-         * addr keeps the address of the last successfully-inserted mapping,
-         * while apply_p2m_changes() considers an address range which is
-         * exclusive of end_gpaddr: add level_size to addr to obtain the
-         * right end of the range
-         */
-        apply_p2m_changes(d, REMOVE,
-                          start_gpaddr, addr + level_sizes[level], orig_maddr,
-                          mattr, 0, p2m_invalid, d->arch.p2m.default_access);
-    }
-
     for ( level = P2M_ROOT_LEVEL; level < 4; level ++ )
     {
         if ( mappings[level] )
@@ -1128,6 +1113,18 @@ out:
 
     spin_unlock(&p2m->lock);
 
+    if ( rc < 0 && ( op == INSERT || op == ALLOCATE ) &&
+         addr != start_gpaddr )
+    {
+        BUG_ON(addr == end_gpaddr);
+        /*
+         * addr keeps the address of the end of the last successfully-inserted
+         * mapping.
+         */
+        apply_p2m_changes(d, REMOVE, start_gpaddr, addr, orig_maddr,
+                          mattr, 0, p2m_invalid, d->arch.p2m.default_access);
+    }
+
     return rc;
 }
 
@@ -1285,7 +1282,8 @@ void p2m_teardown(struct domain *d)
     while ( (pg = page_list_remove_head(&p2m->pages)) )
         free_domheap_page(pg);
 
-    free_domheap_pages(p2m->root, P2M_ROOT_ORDER);
+    if ( p2m->root )
+        free_domheap_pages(p2m->root, P2M_ROOT_ORDER);
 
     p2m->root = NULL;
 
@@ -1458,32 +1456,25 @@ err:
     return page;
 }
 
-struct page_info *get_page_from_gva(struct domain *d, vaddr_t va,
+struct page_info *get_page_from_gva(struct vcpu *v, vaddr_t va,
                                     unsigned long flags)
 {
+    struct domain *d = v->domain;
     struct p2m_domain *p2m = &d->arch.p2m;
     struct page_info *page = NULL;
     paddr_t maddr = 0;
     int rc;
 
+    /*
+     * XXX: To support a different vCPU, we would need to load the
+     * VTTBR_EL2, TTBR0_EL1, TTBR1_EL1 and SCTLR_EL1
+     */
+    if ( v != current )
+        return NULL;
+
     spin_lock(&p2m->lock);
 
-    if ( unlikely(d != current->domain) )
-    {
-        unsigned long irq_flags;
-
-        local_irq_save(irq_flags);
-        p2m_load_VTTBR(d);
-
-        rc = gvirt_to_maddr(va, &maddr, flags);
-
-        p2m_load_VTTBR(current->domain);
-        local_irq_restore(irq_flags);
-    }
-    else
-    {
-        rc = gvirt_to_maddr(va, &maddr, flags);
-    }
+    rc = gvirt_to_maddr(va, &maddr, flags);
 
     if ( rc )
         goto err;
