@@ -295,6 +295,7 @@ static int msixtbl_write(struct vcpu *v, unsigned long address,
         if ( len != 8 || !index )
             goto out;
         val >>= 32;
+        address += 4;
     }
 
     /* Exit to device model when unmasking and address/data got modified. */
@@ -335,12 +336,64 @@ out:
 static int msixtbl_range(struct vcpu *v, unsigned long addr)
 {
     const struct msi_desc *desc;
+    const ioreq_t *r;
 
     rcu_read_lock(&msixtbl_rcu_lock);
     desc = msixtbl_addr_to_desc(msixtbl_find_entry(v, addr), addr);
     rcu_read_unlock(&msixtbl_rcu_lock);
 
-    return !!desc;
+    if ( desc )
+        return 1;
+
+    r = &v->arch.hvm_vcpu.hvm_io.io_req;
+    if ( r->state != STATE_IOREQ_READY || r->addr != addr )
+        return 0;
+    ASSERT(r->type == IOREQ_TYPE_COPY);
+    if ( r->dir == IOREQ_WRITE )
+    {
+        unsigned int size = r->size;
+
+        if ( !r->data_is_ptr )
+        {
+            uint64_t data = r->data;
+
+            if ( size == 8 )
+            {
+                BUILD_BUG_ON(!(PCI_MSIX_ENTRY_VECTOR_CTRL_OFFSET & 4));
+                data >>= 32;
+                addr += size = 4;
+            }
+            if ( size == 4 &&
+                 ((addr & (PCI_MSIX_ENTRY_SIZE - 1)) ==
+                  PCI_MSIX_ENTRY_VECTOR_CTRL_OFFSET) &&
+                 !(data & PCI_MSIX_VECTOR_BITMASK) )
+            {
+                v->arch.hvm_vcpu.hvm_io.msix_snoop_address = addr;
+                v->arch.hvm_vcpu.hvm_io.msix_snoop_gpa = 0;
+            }
+        }
+        else if ( (size == 4 || size == 8) &&
+                  /* Only support forward REP MOVS for now. */
+                  !r->df &&
+                  /*
+                   * Only fully support accesses to a single table entry for
+                   * now (if multiple ones get written to in one go, only the
+                   * final one gets dealt with).
+                   */
+                  r->count && r->count <= PCI_MSIX_ENTRY_SIZE / size &&
+                  !((addr + (size * r->count)) & (PCI_MSIX_ENTRY_SIZE - 1)) )
+        {
+            BUILD_BUG_ON((PCI_MSIX_ENTRY_VECTOR_CTRL_OFFSET + 4) &
+                         (PCI_MSIX_ENTRY_SIZE - 1));
+
+            v->arch.hvm_vcpu.hvm_io.msix_snoop_address =
+                addr + size * r->count - 4;
+            v->arch.hvm_vcpu.hvm_io.msix_snoop_gpa =
+                r->data + size * r->count - 4;
+        }
+    }
+
+    return 0;
 }
 
 static const struct hvm_mmio_ops msixtbl_mmio_ops = {
@@ -406,9 +459,6 @@ int msixtbl_pt_register(struct domain *d, struct pirq *pirq, uint64_t gtable)
         return r;
     }
 
-    if ( !irq_desc->msi_desc )
-        goto out;
-
     msi_desc = irq_desc->msi_desc;
     if ( !msi_desc )
         goto out;
@@ -433,6 +483,24 @@ found:
 out:
     spin_unlock_irq(&irq_desc->lock);
     xfree(new_entry);
+
+    if ( !r )
+    {
+        struct vcpu *v;
+
+        for_each_vcpu ( d, v )
+        {
+            if ( (v->pause_flags & VPF_blocked_in_xen) &&
+                 !v->arch.hvm_vcpu.hvm_io.msix_snoop_gpa &&
+                 v->arch.hvm_vcpu.hvm_io.msix_snoop_address ==
+                 (gtable + msi_desc->msi_attrib.entry_nr *
+                           PCI_MSIX_ENTRY_SIZE +
+                  PCI_MSIX_ENTRY_VECTOR_CTRL_OFFSET) )
+                v->arch.hvm_vcpu.hvm_io.msix_unmask_address =
+                    v->arch.hvm_vcpu.hvm_io.msix_snoop_address;
+        }
+    }
+
     return r;
 }
 
@@ -450,9 +518,6 @@ void msixtbl_pt_unregister(struct domain *d, struct pirq *pirq)
     if ( !irq_desc )
         return;
 
-    if ( !irq_desc->msi_desc )
-        goto out;
-
     msi_desc = irq_desc->msi_desc;
     if ( !msi_desc )
         goto out;
@@ -508,6 +573,28 @@ void msixtbl_pt_cleanup(struct domain *d)
 void msix_write_completion(struct vcpu *v)
 {
     unsigned long ctrl_address = v->arch.hvm_vcpu.hvm_io.msix_unmask_address;
+    unsigned long snoop_addr = v->arch.hvm_vcpu.hvm_io.msix_snoop_address;
+
+    v->arch.hvm_vcpu.hvm_io.msix_snoop_address = 0;
+
+    if ( !ctrl_address && snoop_addr &&
+         v->arch.hvm_vcpu.hvm_io.msix_snoop_gpa )
+    {
+        const struct msi_desc *desc;
+        uint32_t data;
+
+        rcu_read_lock(&msixtbl_rcu_lock);
+        desc = msixtbl_addr_to_desc(msixtbl_find_entry(v, snoop_addr),
+                                    snoop_addr);
+        rcu_read_unlock(&msixtbl_rcu_lock);
+
+        if ( desc &&
+             hvm_copy_from_guest_phys(&data,
+                                      v->arch.hvm_vcpu.hvm_io.msix_snoop_gpa,
+                                      sizeof(data)) == HVMCOPY_okay &&
+             !(data & PCI_MSIX_VECTOR_BITMASK) )
+            ctrl_address = snoop_addr;
+    }
 
     if ( !ctrl_address )
         return;
