@@ -187,6 +187,8 @@ struct rt_dom {
     struct domain *dom;         /* pointer to upper domain */
 };
 
+static int rt_cpu_pick(const struct scheduler *ops, struct vcpu *vc);
+
 /*
  * Useful inline functions
  */
@@ -621,17 +623,23 @@ static void
 rt_vcpu_insert(const struct scheduler *ops, struct vcpu *vc)
 {
     struct rt_vcpu *svc = rt_vcpu(vc);
-    s_time_t now = NOW();
+    s_time_t now;
+    spinlock_t *lock;
 
-    /* not addlocate idle vcpu to dom vcpu list */
-    if ( is_idle_vcpu(vc) )
-        return;
+    BUG_ON( is_idle_vcpu(vc) );
 
+    /* This is safe because vc isn't yet being scheduled */
+    vc->processor = rt_cpu_pick(ops, vc);
+
+    lock = vcpu_schedule_lock_irq(vc);
+
+    now = NOW();
     if ( now >= svc->cur_deadline )
         rt_update_deadline(now, svc);
 
     if ( !__vcpu_on_q(svc) && vcpu_runnable(vc) && !vc->is_running )
         __runq_insert(ops, svc);
+    vcpu_schedule_unlock_irq(lock, vc);
 
     /* add rt_vcpu svc to scheduler-specific vcpu list of the dom */
     list_add_tail(&svc->sdom_elem, &svc->sdom->vcpu);
@@ -1032,7 +1040,7 @@ static void
 rt_vcpu_wake(const struct scheduler *ops, struct vcpu *vc)
 {
     struct rt_vcpu * const svc = rt_vcpu(vc);
-    s_time_t now = NOW();
+    s_time_t now;
     struct rt_private *prv = rt_priv(ops);
     struct rt_vcpu *snext = NULL; /* highest priority on RunQ */
     struct rt_dom *sdom = NULL;
@@ -1068,6 +1076,7 @@ rt_vcpu_wake(const struct scheduler *ops, struct vcpu *vc)
         return;
     }
 
+    now = NOW();
     if ( now >= svc->cur_deadline)
         rt_update_deadline(now, svc);
 
