@@ -23,7 +23,6 @@
 #include <xen/pci_regs.h>
 #include <xen/paging.h>
 #include <xen/softirq.h>
-#include <asm/hvm/iommu.h>
 #include <asm/amd-iommu.h>
 #include <asm/hvm/svm/amd-iommu-proto.h>
 #include "../ats.h"
@@ -117,8 +116,7 @@ static void amd_iommu_setup_domain_device(
     int req_id, valid = 1;
     int dte_i = 0;
     u8 bus = pdev->bus;
-
-    struct hvm_iommu *hd = domain_hvm_iommu(domain);
+    const struct domain_iommu *hd = dom_iommu(domain);
 
     BUG_ON( !hd->arch.root_table || !hd->arch.paging_mode ||
             !iommu->dev_table.buffer );
@@ -224,7 +222,7 @@ int __init amd_iov_detect(void)
     return scan_pci_devices();
 }
 
-static int allocate_domain_resources(struct hvm_iommu *hd)
+static int allocate_domain_resources(struct domain_iommu *hd)
 {
     /* allocate root table */
     spin_lock(&hd->arch.mapping_lock);
@@ -259,7 +257,7 @@ static int get_paging_mode(unsigned long entries)
 
 static int amd_iommu_domain_init(struct domain *d)
 {
-    struct hvm_iommu *hd = domain_hvm_iommu(d);
+    struct domain_iommu *hd = dom_iommu(d);
 
     /* allocate page directroy */
     if ( allocate_domain_resources(hd) != 0 )
@@ -341,7 +339,7 @@ void amd_iommu_disable_domain_device(struct domain *domain,
         AMD_IOMMU_DEBUG("Disable: device id = %#x, "
                         "domain = %d, paging mode = %d\n",
                         req_id,  domain->domain_id,
-                        domain_hvm_iommu(domain)->arch.paging_mode);
+                        dom_iommu(domain)->arch.paging_mode);
     }
     spin_unlock_irqrestore(&iommu->lock, flags);
 
@@ -358,7 +356,7 @@ static int reassign_device(struct domain *source, struct domain *target,
 {
     struct amd_iommu *iommu;
     int bdf;
-    struct hvm_iommu *t = domain_hvm_iommu(target);
+    struct domain_iommu *t = dom_iommu(target);
 
     bdf = PCI_BDF2(pdev->bus, pdev->devfn);
     iommu = find_iommu_for_device(pdev->seg, bdf);
@@ -459,7 +457,7 @@ static void deallocate_page_table(struct page_info *pg)
 
 static void deallocate_iommu_page_tables(struct domain *d)
 {
-    struct hvm_iommu *hd  = domain_hvm_iommu(d);
+    struct domain_iommu *hd = dom_iommu(d);
 
     if ( iommu_use_hap_pt(d) )
         return;
@@ -599,7 +597,7 @@ static void amd_dump_p2m_table_level(struct page_info* pg, int level,
 
 static void amd_dump_p2m_table(struct domain *d)
 {
-    struct hvm_iommu *hd  = domain_hvm_iommu(d);
+    const struct domain_iommu *hd = dom_iommu(d);
 
     if ( !hd->arch.root_table )
         return;
